
#include "bootloader_common.h"

#include <avr/io.h>
#include <avr/boot.h>
#include <avr/interrupt.h>

// ----------------------------------------------------------------------------

#if 	defined(__AVR_ATmega8__) || \
		defined(__AVR_ATmega16__) || \
		defined(__AVR_ATmega32__)
	
	#define	IV_REG	GICR
#elif 	defined(__AVR_ATmega88__) || \
		defined(__AVR_ATmega168__) || \
		defined(__AVR_ATmega644__) || \
		defined(__AVR_AT90CAN32__) || \
		defined(__AVR_AT90CAN64__) || \
		defined(__AVR_AT90CAN128__)
	
	#define	IV_REG	MCUCR
#else
	#error	chosen AVR type is not supported yet!
#endif

// ----------------------------------------------------------------------------
// starts the application program

void boot_jump_to_application(void)
{
	// disable interrupts
	cli();
	
	// relocate interrupt vectors
	uint8_t reg = IV_REG & ~((1 << IVCE) | (1 << IVSEL));
	
	IV_REG = reg | (1 << IVCE);
	IV_REG = reg;
	
	// jump to address 0x0000
	#if FLASHEND > 0xffff
	asm volatile (	"push r1" "\n\t"
					"push r1" "\n\t"
					"push r1" "\n\t"
					::);
	#else
	asm volatile (	"push r1" "\n\t"
					"push r1" "\n\t"
					::);
	#endif
}

// ----------------------------------------------------------------------------
// write a complete page to the flash memorey

void boot_program_page(uint16_t page, uint8_t *buf)
{
	uint32_t adr = page * SPM_PAGESIZE;
	
	// Disable interrupts.
	uint8_t sreg = SREG;
	cli();
	
	boot_page_erase(adr);
	boot_spm_busy_wait();	  // Wait until the memory is erased.
	
	for (uint16_t i=0; i < SPM_PAGESIZE; i+=2)
	{
		// Set up little-endian word.
		
		uint16_t w = *buf++;
		w += (*buf++) << 8;
		
		boot_page_fill(adr + i, w);
	}
	
	boot_page_write(adr);		// Store buffer in flash page.
	boot_spm_busy_wait();		// Wait until the memory is written.
	
	// Reenable RWW-section again. We need this if we want to jump back
	// to the application after bootloading.
	boot_rww_enable();
	
	// Re-enable interrupts.
	SREG = sreg;
}
