#ifndef	DEFAULTS_H
#define	DEFAULTS_H

#include "config.h"

// ----------------------------------------------------------------------------
// create pagesize identifier

#if	SPM_PAGESIZE == 32
	#define	PAGESIZE_IDENTIFIER		(0)
#elif SPM_PAGESIZE == 64
	#define	PAGESIZE_IDENTIFIER		(1)
#elif SPM_PAGESIZE == 128
	#define	PAGESIZE_IDENTIFIER		(2)
#elif SPM_PAGESIZE == 256
	#define	PAGESIZE_IDENTIFIER		(3)
#else
	#error	Strange value for SPM_PAGESIZE. Check the define!
#endif

// -----------------------------------------------------------------------------
// check defines for the bootloader led

#ifdef BOOT_LED

	#ifndef	BOOT_LED_SET_OUTPUT	
		#define	BOOT_LED_SET_OUTPUT		SET_OUTPUT(BOOT_LED)
	#endif
	
	#ifndef	BOOT_LED_ON
		#define	BOOT_LED_ON				SET(BOOT_LED)
	#endif
	
	#ifndef	BOOT_LED_OFF
		#define	BOOT_LED_OFF			RESET(BOOT_LED)
	#endif
	
	#ifndef	BOOT_LED_TOGGLE	
		#define	BOOT_LED_TOGGLE			TOGGLE(BOOT_LED)
	#endif

#else
	
	#define	BOOT_LED_SET_OUTPUT			
	#define	BOOT_LED_ON
	#define	BOOT_LED_OFF
	#define	BOOT_LED_TOGGLE
	
	#warning	compiling bootloader without LED support

#endif

// -----------------------------------------------------------------------------
#ifndef	BOOT_INIT
	#define	BOOT_INIT
#endif

#ifndef	BOOTLOADER_TYPE
	#define	BOOTLOADER_TYPE		0
#endif

// -----------------------------------------------------------------------------
// set current version of the bootloader

#define	BOOTLOADER_VERSION		2

// -----------------------------------------------------------------------------
// CAN settings

#ifndef	SPI_PRESCALER
	#define	SPI_PRESCALER		8
#endif

#ifndef	MCP2515_BITRATE
	#define	MCP2515_BITRATE		125
#endif

#ifndef	MCP2515_INTERRUPTS
	#define	MCP2515_INTERRUPTS	(1<<RX1IE)|(1<<RX0IE)
#endif

#ifndef	MCP2515_CLKOUT_PRESCALER
	#define	MCP2515_CLKOUT_PRESCALER	0
#endif

// ----------------------------------------------------------------------------
// einige AVR spezifische Daten abfragen

#if defined(__AVR_ATmega8__)
	// $0C00
	#define	RWW_PAGES	96
	#define	RAMSTART	0x0060
	#define	SIG_FAMILY	0x93
	#define	SIG_DEVICE	0x07
	#define	TIMER_INTERRUPT_FLAG_REGISTER	TIFR
	
#elif defined(__AVR_ATmega88__)
	// $0C00
	#define	RWW_PAGES	96
	#define	RAMSTART	0x0100
	#define	SIG_FAMILY	0x93
	#define	SIG_DEVICE	0x0A
	#define	TIMER_INTERRUPT_FLAG_REGISTER	TIFR1
	
#elif defined(__AVR_ATmega168__)
	// $1C00
	#define	RWW_PAGES	112
	#define	RAMSTART	0x0100
	#define	SIG_FAMILY	0x94
	#define	SIG_DEVICE	0x06
	#define	TIMER_INTERRUPT_FLAG_REGISTER	TIFR1
	
#elif defined(__AVR_ATmega16__)	// TODO
	// $1C00
	#define	RWW_PAGES	112
	#define	RAMSTART	0x0060
	#define	SIG_FAMILY	0x94
	#define	SIG_DEVICE	0x03
	#define	TIMER_INTERRUPT_FLAG_REGISTER	TIFR
	
#elif defined(__AVR_ATmega32__)	// TODO
	// $3800
	#define	RWW_PAGES	224
	#define	RAMSTART	0x0060
	#define	SIG_FAMILY	0x95
	#define	SIG_DEVICE	0x02
	#define	TIMER_INTERRUPT_FLAG_REGISTER	TIFR
	
#elif defined(__AVR_ATmega644__)	// TODO
	// $7000
	#define	RWW_PAGES	224
	#define	RAMSTART	0x0100
	#define	SIG_FAMILY	0x96
	#define	SIG_DEVICE	0x09
	#define	TIMER_INTERRUPT_FLAG_REGISTER	TIFR1
	
#else
	#error	chosen AVR command is not supported yet!
#endif

// ----------------------------------------------------------------------------
// Timereinstellung fuer aktuelle Taktfrequenz auswaehlen (500ms)
//
// TIMER_PRELOAD = 65536 - (0.5s * F_CPU) / 1024

#if F_CPU == 4000000UL
	#define	TIMER_PRESCALER		(1<<CS12)|(1<<CS10)		// Prescaler = 1024
	#define TIMER_PRELOAD		63583
#elif F_CPU == 7372800UL
	#define	TIMER_PRESCALER		(1<<CS12)|(1<<CS10)		// Prescaler = 1024
	#define	TIMER_PRELOAD		61936
#elif F_CPU == 8000000UL
	#define	TIMER_PRESCALER		(1<<CS12)|(1<<CS10)		// Prescaler = 1024
	#define	TIMER_PRELOAD		61629
#elif F_CPU == 14745600UL
	#define	TIMER_PRESCALER		(1<<CS12)|(1<<CS10)		// Prescaler = 1024
	#define	TIMER_PRELOAD		58336
#elif F_CPU == 16000000UL
	#define	TIMER_PRESCALER		(1<<CS12)|(1<<CS10)		// Prescaler = 1024
	#define	TIMER_PRELOAD		57724
#elif F_CPU == 20000000UL
	#define	TIMER_PRESCALER		(1<<CS12)|(1<<CS10)		// Prescaler = 1024
	#define	TIMER_PRELOAD		55770
#else
	#error	choosen F_CPU not supported yet!
#endif


#endif	// DEFAULTS_H
