
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "utils.h"
#include "simple_mcp2515.h"


#if defined(__AVR_ATmega16__) || defined(__AVR_ATmega32__) || \
	defined(__AVR_ATmega644__)
	#define	P_MOSI	B,5
	#define	P_MISO	B,6
	#define	P_SCK	B,7
	
	#define	PORT_SPI	PORTB
	#define	DDR_SPI		DDRB
#elif defined(__AVR_ATmega8__)  || defined(__AVR_ATmega48__) || \
	  defined(__AVR_ATmega88__) || defined(__AVR_ATmega168__)
	#define	P_MOSI	B,3
	#define	P_MISO	B,4
	#define	P_SCK	B,5
	
	#define	PORT_SPI	PORTB
	#define	DDR_SPI		DDRB
#else
	#error	the choosen AVR Type is not yet supported by this library.
#endif

#ifndef	MCP2515_CS
	#error	MCP2515_CS ist nicht definiert!
#endif

#ifdef	SPI_PRESCALER
	#if (SPI_PRESCALER == 2) || (SPI_PRESCALER == 8) || (SPI_PRESCALER == 32) || (SPI_PRESCALER == 64)
		#define	R_SPSR	(1<<SPI2X)
		#define SPI_PRESCALER_ 	(SPI_PRESCALER * 2)
	#else
		#define	R_SPSR	0
		#define	SPI_PRESCALER_	SPI_PRESCALER
	#endif
	
	#if (SPI_PRESCALER_ == 4)
		#define	R_SPCR	(0<<SPR1)|(0<<SPR0)
	#elif (SPI_PRESCALER_ == 16)
		#define	R_SPCR	(0<<SPR1)|(1<<SPR0)
	#elif (SPI_PRESCALER_ == 64)
		#define	R_SPCR	(1<<SPR1)|(0<<SPR0)
	#elif (SPI_PRESCALER_ == 128)
		#define	R_SPCR	(1<<SPR1)|(1<<SPR0)
	#else
		#error	 SPI_PRESCALER must be one of the values of 2^n with n = 2..7!
	#endif
#else
	#error	SPI_PRESCALER not defined!
#endif

#ifndef	MCP2515_CLKOUT_PRESCALER
	#error	MCP2515_CLKOUT_PRESCALER not defined!
#elif MCP2515_CLKOUT_PRESCALER == 0
	#define	CLKOUT_PRESCALER_	0x0
#elif MCP2515_CLKOUT_PRESCALER == 1
	#define	CLKOUT_PRESCALER_	0x4
#elif MCP2515_CLKOUT_PRESCALER == 2
	#define	CLKOUT_PRESCALER_	0x5
#elif MCP2515_CLKOUT_PRESCALER == 4
	#define	CLKOUT_PRESCALER_	0x6
#elif MCP2515_CLKOUT_PRESCALER == 8
	#define	CLKOUT_PRESCALER_	0x7
#else
	#error	invaild value of MCP2515_CLKOUT_PRESCALER
#endif

#ifndef	MCP2515_BITRATE
	#error	MCP2515_BITRATE not defined!
#else
	#if	MCP2515_BITRATE == 125
		// 125 kbps
		#define	R_CNF3	((1<<PHSEG21))
		#define	R_CNF2	((1<<BTLMODE)|(1<<PHSEG11))
		#define	R_CNF1	((1<<BRP2)|(1<<BRP1)|(1<<BRP0))
	#elif MCP2515_BITRATE == 250
		// 250 kbps
		#define	R_CNF3	((1<<PHSEG21))
		#define	R_CNF2	((1<<BTLMODE)|(1<<PHSEG11))
		#define	R_CNF1	((1<<BRP1)|(1<<BRP0))
	#elif MCP2515_BITRATE == 500
		// 500 kbps
		#define	R_CNF3	((1<<PHSEG21))
		#define	R_CNF2	((1<<BTLMODE)|(1<<PHSEG11))
		#define	R_CNF1	((1<<BRP0))
	#elif MCP2515_BITRATE == 1000
		// 1 Mbps
		#define	R_CNF3	((1<<PHSEG21))
		#define	R_CNF2	((1<<BTLMODE)|(1<<PHSEG11))
		#define	R_CNF1	(0)
	#else
		#error invalid value for MCP2515_BITRATE
	#endif
#endif

// ----------------------------------------------------------------------------
extern uint8_t spi_putc( uint8_t data );

// ----------------------------------------------------------------------------
extern void mcp2515_write_register( uint8_t adress, uint8_t data );

// -------------------------------------------------------------------------
static uint8_t PROGMEM mcp2515_register_map[45] = {
	0xff, 0xc0, 0x00, 0x00,				// Filter 0
	0xff, 0xc0, 0x00, 0x00,				// Filter 1
	0xff, 0xc0, 0x00, 0x00,				// Filter 2
	0,									// BFPCTRL
	0,									// TXRTSCTRL
	0,									// CANSTAT (read-only)
	(1<<REQOP2) | CLKOUT_PRESCALER_,	// CANCTRL
	0xff, 0xc0, 0x00, 0x00,				// Filter 3
	0xff, 0xc0, 0x00, 0x00,				// Filter 4
	0xff, 0xc0, 0x00, 0x00,				// Filter 5
	0,									// TEC (read-only)
	0,									// REC (read-only)
	0,									// CANSTAT (read-only)
	(1<<REQOP2) | CLKOUT_PRESCALER_,	// CANCTRL
	0xff, 0xc0, 0x00, 0x00,				// Mask 0
	0xff, 0xc0, 0x00, 0x00,				// Mask 1
	R_CNF3,
	R_CNF2,
	R_CNF1,
	MCP2515_INTERRUPTS,
	0							// clear interrupt flags
};

// ----------------------------------------------------------------------------
// abgespeckte Variante der "normalen" MCP2515 Initialisierung

void mcp2515_init(void)
{
	// Aktivieren der Pins fuer das SPI Interface
	PORT_SPI &=  ~((1 << PIN_NUM(P_SCK)) | (1 << PIN_NUM(P_MOSI)));
	DDR_SPI |= (1 << PIN_NUM(P_SCK)) | (1 << PIN_NUM(P_MOSI));
	
	SET(MCP2515_CS);
	SET_OUTPUT(MCP2515_CS);
	
	// Aktivieren des SPI Master Interfaces
	SPCR = (1 << SPE) | (1 << MSTR) | R_SPCR;
	SPSR = R_SPSR;
	
	_delay_us(1);
	
	// MCP2515 per Software Reset zuruecksetzten,
	// danach ist er automatisch im Konfigurations Modus
	RESET(MCP2515_CS);
	spi_putc(SPI_RESET);
	SET(MCP2515_CS);
	
	// ein bisschen warten bis der MCP2515 sich neu gestartet hat
	_delay_ms(0.1);
	
	// Filter usw. setzen
	RESET(MCP2515_CS);
	spi_putc(SPI_WRITE);
	spi_putc(RXF0SIDH);
	for (uint8_t i = 0; i < sizeof(mcp2515_register_map); i++)
		spi_putc(pgm_read_byte(&mcp2515_register_map[i]));
	SET(MCP2515_CS);
	
	// nur Standard IDs, Message Rollover nach Puffer 1
	mcp2515_write_register(RXB0CTRL, (0<<RXM1)|(1<<RXM0)|(1<<BUKT));
	mcp2515_write_register(RXB1CTRL, (0<<RXM1)|(1<<RXM0));
	
	// MCP2515 zurueck in den normalen Modus versetzten
	mcp2515_write_register(CANCTRL, CLKOUT_PRESCALER_);
}
